@@ -1,6 +1,7 @@
 /* JitterLisp: global settings: header.
 
    Copyright (C) 2017, 2018, 2019 Luca Saiu
+   Updated in 2023 by Luca Saiu
    Written by Luca Saiu
 
    This file is part of the JitterLisp language implementation, distributed as
@@ -17,12 +18,13 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Jitter.  If not, see <http://www.gnu.org/licenses/>. */
+   along with GNU Jitter.  If not, see <https://www.gnu.org/licenses/>. */
 
 
 #ifndef JITTERLISP_SETTINGS_H_
 #define JITTERLISP_SETTINGS_H_
 
+#include <jitter/jitter.h>
 #include <stdbool.h>
 
 #include <jitter/jitter-dynamic-buffer.h>
