@@ -1,6 +1,6 @@
 /* VM library: specializer header file.
 
-   Copyright (C) 2016, 2017, 2019, 2020 Luca Saiu
+   Copyright (C) 2016, 2017, 2019, 2020, 2022 Luca Saiu
    Written by Luca Saiu
 
    This file is part of GNU Jitter.
@@ -16,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Jitter.  If not, see <http://www.gnu.org/licenses/>. */
+   along with GNU Jitter.  If not, see <https://www.gnu.org/licenses/>. */
 
 
 #ifndef JITTER_SPECIALIZE_H_
@@ -85,7 +85,7 @@ struct jitter_executable_routine
 #if   (defined(JITTER_DISPATCH_SWITCH)                 \
        || defined(JITTER_DISPATCH_DIRECT_THREADING)    \
        || defined(JITTER_DISPATCH_MINIMAL_THREADING))
-  union jitter_specialized_word *specialized_program;
+  union jitter_specialized_word *specialized_routine;
 #elif defined(JITTER_DISPATCH_NO_THREADING)
   /* Nothing. */
 #elif ! defined (JITTER_INTERNAL)
@@ -154,57 +154,6 @@ void
 jitter_unpin_executable_routine (struct jitter_executable_routine *er);
 
 
-
-
-/* Program points.
- * ************************************************************************** */
-
-/* The type of a program point at run time in an executable routine.  This is
-   the type of object than can be passed to JITTER_BRANCH , and its actual
-   definition depends on the dispatch.  Notice that however, in every
-   case, a program point is a pointer-to-constant type and therefore fits in a
-   word. */
-#ifdef JITTER_DISPATCH_NO_THREADING
-  /* With no-threading dispatch a program point is the address of a machine
-     instruction -- from C, it's what a goto * statement accepts.  I don't
-     need to worry about non-GCC compilers, since no-threading relies on GCC
-     extensions. */
-  typedef const void *
-  jitter_program_point;
-#else
-  /* On every other dispatch a program point is a pointer to a word
-     in the executable routine -- in the case of switch dispatching that word
-     will contain a specialized opcode, with threading it will contain the
-     address of a machine instruction (see the case above) followed by the
-     VM instruction arguments. */
-  typedef const union jitter_word *
-  jitter_program_point;
-#endif // #ifdef JITTER_DISPATCH_NO_THREADING
-
-/* Expand to an expression evaluating to the program point of the first
-   instruction in the pointed executable routine, as some object which is
-   correct to pass to JITTER_BRANCH.  The expression type will be
-   jitter_program_point.
-
-   This macro is usable from within VM instructions, which might for example
-   perform a jump to the beginning of a different VM routine.  The expansion of
-   this macro is guaranteed not to contain function calls, and is safe to use
-   without extra C function wrappers. */
-#ifdef JITTER_DISPATCH_NO_THREADING
-  /* In this case the executable routine contains a separate pointer to the
-     beginning of the executable region for the native routine.  Of course
-     the first instruction is at the beginning of the region. */
-# define JITTER_EXECUTABLE_ROUTINE_BEGINNING(_jitter_executable_routine_ptr)  \
-    ((_jitter_executable_routine_ptr)->native_code)
-#else
-  /* With switch dispatching or threading the first program point is a pointer
-     to the beginning of the specialized routine array. */
-# define JITTER_EXECUTABLE_ROUTINE_BEGINNING(_jitter_executable_routine_ptr)  \
-    ((jitter_program_point)                                                   \
-     ((_jitter_executable_routine_ptr)->specialized_program))
-#endif // ifdef JITTER_DISPATCH_NO_THREADING
-
-
 
 
 /* Internal declarations.
@@ -227,7 +176,7 @@ enum jitter_specialized_instruction_opcode
     jitter_specialized_instruction_opcode_PRETENDTOJUMPANYWHERE = 7,
   };
 
-/* Add an opcode to the specialized program which is being built.  This is an
+/* Add an opcode to the specialised routine which is being built.  This is an
    auxiliary function used by vmprefix_specialize_instruction .  The opcode type
    should actually be the VM-dependent enum
    vmprefix_specialized_instruction_opcode , but it's safe to use a sufficiently
@@ -239,13 +188,13 @@ jitter_add_specialized_instruction_opcode
        the type is VM-dependent. */
     jitter_uint opcode);
 
-/* Add a fixnum literal to the specialized program which is being built.  This
+/* Add a fixnum literal to the specialised routine which is being built.  This
    is an auxiliary function used by vmprefix_specialize_instruction . */
 void
 jitter_add_specialized_instruction_literal (struct jitter_mutable_routine *p,
                                             jitter_uint literal);
 
-/* Add a label literal (as an instruction index) to the specialized program
+/* Add a label literal (as an instruction index) to the specialised routine
    which is being built.  This is an auxiliary function used by
    vmprefix_specialize_instruction . */
 void
