@@ -1,6 +1,7 @@
 /* VM library: native code patching for RISC-V .
 
    Copyright (C) 2017, 2019, 2020 Luca Saiu
+   Updated in 2022 and 2023 by Luca Saiu
    Written by Luca Saiu
 
    This file is part of GNU Jitter.
@@ -16,13 +17,20 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Jitter.  If not, see <http://www.gnu.org/licenses/>. */
+   along with GNU Jitter.  If not, see <https://www.gnu.org/licenses/>. */
 
 
 #include <stdint.h>
 #include <assert.h>
 #include <string.h>
 
+#if defined (JITTER_HAVE___RISCV_FLUSH_ICACHE)
+# include <sys/cachectl.h>  /* for __riscv_flush_icache */
+#else
+  /* Nothing serious: the generic GCC builtin __builtin___clear_cache should be
+     sufficient. */
+#endif
+
 #include <jitter/jitter-fatal.h>
 
 #include <jitter/jitter-arithmetic.h>
@@ -122,8 +130,11 @@ struct jitter_riscv_b_type_instruction
 void
 jitter_invalidate_icache (char *from, size_t byte_no)
 {
-  /* I am assuming that this does not need to do anything on RISC-V.  Not
-     tested on real hardware yet. */
+#if defined (JITTER_HAVE___RISCV_FLUSH_ICACHE)
+  __riscv_flush_icache (from, from + byte_no, 0);
+#else
+  /* Do nothing: __builtin___clear_cache is called elsewhere. */
+#endif
 }
 
 /* Return the distance in bytes, from the pointed instruction, to be patched,
@@ -135,7 +146,14 @@ static jitter_int
 jitter_distance_from (const char *destination_p,
                       const char *branch_instruction_p)
 {
-  return destination_p - branch_instruction_p;
+  return (/* To avoid undefined behaviour subtract two unsigned integers --
+             which may wrap around -- and then convert the result, which will be
+             on two's complement on this platform, to a signed value.  We are
+             still technically in undefined behaviour's territory because the
+             two pointers do not belong to the same buffer, but GCC can handle
+             this. */
+          (jitter_int)
+          ((jitter_uint) destination_p - (jitter_uint) branch_instruction_p));
 }
 
 enum jitter_snippet_to_patch
