@@ -168,7 +168,7 @@ ios_dev_stream_pread (void *iod, void *buf, size_t count, ios_dev_off offset)
   if (ios_buffer_get_begin_offset (buffer) > offset)
     return IOD_EOF;
 
-  /* If the requsted range is in the buffer, return it. */
+  /* If the requested range is in the buffer, return it. */
   if (ios_buffer_get_end_offset (buffer) >= offset + count)
     return ios_buffer_pread (buffer, buf, count, offset);
 
@@ -249,12 +249,14 @@ static int
 ios_dev_stream_flush (void *iod, ios_dev_off offset)
 {
   struct ios_dev_stream *sio = iod;
+
   if (sio->flags & IOS_F_READ
       && offset > ios_buffer_get_begin_offset (sio->buffer)
       && offset <= ios_buffer_get_end_offset (sio->buffer))
     return ios_buffer_forget_till (sio->buffer, offset);
-  else
-    return IOS_OK;
+  else if (sio->flags & IOS_F_WRITE)
+    fflush (sio->file);
+  return IOS_OK;
 }
 
 struct ios_dev_if ios_dev_stream =
