@@ -60,7 +60,6 @@ enum pkl_ast_code
   PKL_AST_FORMAT,
   PKL_AST_FORMAT_ARG,
   PKL_AST_INCRDECR,
-  PKL_AST_GCD,
   PKL_AST_ASM_EXP,
   PKL_AST_LAST_EXP = PKL_AST_ASM_EXP,
   /* Types.  */
@@ -164,6 +163,7 @@ enum pkl_ast_type_code
   PKL_TYPE_FUNCTION,
   PKL_TYPE_OFFSET,
   PKL_TYPE_ANY,
+  PKL_TYPE_ALIAS,
   PKL_TYPE_NOTYPE,
 };
 
@@ -889,13 +889,23 @@ pkl_ast_node pkl_ast_make_func_type_arg (pkl_ast ast,
    If NAME is not NULL, then this specific type instance has a given
    name, which is encoded in a PKL_AST_IDENTIFIER node.
 
-   CODE contains the kind of type, as defined in the pkl_ast_type_code
-   enumeration above.
+   CODE contains the kind of the type, as defined in the
+   pkl_ast_type_code enumeration above, perhaps after a chain of alias
+   types.  CODE2 is like CODE but it refers to the code of the
+   immediate type, without traversing, which may be a type alias.
 
    COMPILED contains an integer identifying the last compiler pass
    that was run on the node.  This is only used in certain nodes, and
    is controlled in the pass manager.
 
+   NAMED_P contains a flag that tells whether this type node is
+   aliased, i.e. it has appeared as an initial in a declaration.  This
+   means this node can only be referred via an alias.
+
+   In alias types, NAME is the name of the aliased type encoded in an
+   IDENTIFIER node.  TYPE is the aliased type.  NEXT_IN_DECL is the
+   next alias node that refers to the same type declaration or NULL.
+
    In integral types, SIGNED is 1 if the type denotes a signed numeric
    type.  In non-integral types SIGNED_P is 0.  SIZE is the size in
    bits of type, which shall be of the supported range (like [1,64]
@@ -928,7 +938,8 @@ pkl_ast_node pkl_ast_make_func_type_arg (pkl_ast ast,
    In offset types, BASE_TYPE is a PKL_AST_TYPE with the base type for
    the offset's magnitude, and UNIT is either a PKL_AST_IDENTIFIER
    containing one of few recognized keywords (b, B, Kb, etc) or a
-   PKL_AST_TYPE.
+   PKL_AST_TYPE.  If a REF_TYPE is specified, then it is the type of
+   the values referred by the offset value; this is akin to a pointer.
 
    In function types, NARG is the number of formal arguments in the
    function type.  ARGS is a chain of PKL_AST_FUNC_TYPE_ARG nodes.
@@ -948,46 +959,53 @@ pkl_ast_node pkl_ast_make_func_type_arg (pkl_ast ast,
    "fallible".  Examples of such constraints are constraint
    expressions in struct type fields or size bounders in arrays.  */
 
-#define PKL_AST_TYPE_CODE(AST) ((AST)->type.code)
-#define PKL_AST_TYPE_NAME(AST) ((AST)->type.name)
-#define PKL_AST_TYPE_COMPLETE(AST) ((AST)->type.complete)
-#define PKL_AST_TYPE_FALLIBLE(AST) ((AST)->type.fallible)
-#define PKL_AST_TYPE_COMPILED(AST) ((AST)->type.compiled)
-#define PKL_AST_TYPE_I_SIZE(AST) ((AST)->type.val.integral.size)
-#define PKL_AST_TYPE_I_SIGNED_P(AST) ((AST)->type.val.integral.signed_p)
-#define PKL_AST_TYPE_I_DYN_P(AST) ((AST)->type.val.integral.dyn_p)
-#define PKL_AST_TYPE_A_BOUND(AST) ((AST)->type.val.array.bound)
-#define PKL_AST_TYPE_A_ETYPE(AST) ((AST)->type.val.array.etype)
-#define PKL_AST_TYPE_A_CLOSURES(AST) ((AST)->type.val.array.closures)
-#define PKL_AST_TYPE_A_MAPPER(AST) ((AST)->type.val.array.closures[0])
-#define PKL_AST_TYPE_A_WRITER(AST) ((AST)->type.val.array.closures[1])
-#define PKL_AST_TYPE_A_BOUNDER(AST) ((AST)->type.val.array.closures[2])
-#define PKL_AST_TYPE_A_CONSTRUCTOR(AST) ((AST)->type.val.array.closures[3])
-#define PKL_AST_TYPE_A_INTEGRATOR(AST) ((AST)->type.val.array.closures[4])
-#define PKL_AST_TYPE_S_NFIELD(AST) ((AST)->type.val.sct.nfield)
-#define PKL_AST_TYPE_S_NCFIELD(AST) ((AST)->type.val.sct.ncfield)
-#define PKL_AST_TYPE_S_NDECL(AST) ((AST)->type.val.sct.ndecl)
-#define PKL_AST_TYPE_S_NELEM(AST) ((AST)->type.val.sct.nelem)
-#define PKL_AST_TYPE_S_ELEMS(AST) ((AST)->type.val.sct.elems)
-#define PKL_AST_TYPE_S_PINNED_P(AST) ((AST)->type.val.sct.pinned_p)
-#define PKL_AST_TYPE_S_UNION_P(AST) ((AST)->type.val.sct.union_p)
-#define PKL_AST_TYPE_S_CLOSURES(AST) ((AST)->type.val.sct.closures)
-#define PKL_AST_TYPE_S_MAPPER(AST) ((AST)->type.val.sct.closures[0])
-#define PKL_AST_TYPE_S_WRITER(AST) ((AST)->type.val.sct.closures[1])
-#define PKL_AST_TYPE_S_CONSTRUCTOR(AST) ((AST)->type.val.sct.closures[2])
-#define PKL_AST_TYPE_S_COMPARATOR(AST) ((AST)->type.val.sct.closures[3])
-#define PKL_AST_TYPE_S_INTEGRATOR(AST) ((AST)->type.val.sct.closures[4])
-#define PKL_AST_TYPE_S_DEINTEGRATOR(AST) ((AST)->type.val.sct.closures[5])
-#define PKL_AST_TYPE_S_TYPIFIER(AST) ((AST)->type.val.sct.closures[6])
-#define PKL_AST_TYPE_S_ITYPE(AST) ((AST)->type.val.sct.itype)
-#define PKL_AST_TYPE_O_UNIT(AST) ((AST)->type.val.off.unit)
-#define PKL_AST_TYPE_O_BASE_TYPE(AST) ((AST)->type.val.off.base_type)
-#define PKL_AST_TYPE_O_REF_TYPE(AST) ((AST)->type.val.off.ref_type)
-#define PKL_AST_TYPE_F_RTYPE(AST) ((AST)->type.val.fun.rtype)
-#define PKL_AST_TYPE_F_NARG(AST) ((AST)->type.val.fun.narg)
-#define PKL_AST_TYPE_F_ARGS(AST) ((AST)->type.val.fun.args)
-#define PKL_AST_TYPE_F_VARARG(AST) ((AST)->type.val.fun.vararg)
-#define PKL_AST_TYPE_F_FIRST_OPT_ARG(AST) ((AST)->type.val.fun.first_opt_arg)
+#define PKL_AST_TYPE_NAME(AST) \
+  ((AST)->type.code == PKL_TYPE_ALIAS ? (AST)->type.val.alias.name : NULL)
+
+#define PKL_AST_TYPE_CODE2(AST) ((AST)->type.code)
+#define PKL_AST_TYPE_CODE(AST) (pkl_ast_type_resolv (AST)->type.code)
+#define PKL_AST_TYPE_COMPLETE(AST) (pkl_ast_type_resolv (AST)->type.complete)
+#define PKL_AST_TYPE_FALLIBLE(AST) (pkl_ast_type_resolv (AST)->type.fallible)
+#define PKL_AST_TYPE_COMPILED(AST) (pkl_ast_type_resolv (AST)->type.compiled)
+#define PKL_AST_TYPE_NAMED_P(AST) (pkl_ast_type_resolv (AST)->type.named_p)
+#define PKL_AST_TYPE_L_NAME(AST) ((AST)->type.val.alias.name)
+#define PKL_AST_TYPE_L_TYPE(AST) ((AST)->type.val.alias.type)
+#define PKL_AST_TYPE_L_NEXT_IN_DECL(AST) ((AST)->type.val.alias.next_in_decl)
+#define PKL_AST_TYPE_I_SIZE(AST) (pkl_ast_type_resolv (AST)->type.val.integral.size)
+#define PKL_AST_TYPE_I_SIGNED_P(AST) (pkl_ast_type_resolv (AST)->type.val.integral.signed_p)
+#define PKL_AST_TYPE_I_DYN_P(AST) (pkl_ast_type_resolv (AST)->type.val.integral.dyn_p)
+#define PKL_AST_TYPE_A_BOUND(AST) (pkl_ast_type_resolv (AST)->type.val.array.bound)
+#define PKL_AST_TYPE_A_ETYPE(AST) (pkl_ast_type_resolv (AST)->type.val.array.etype)
+#define PKL_AST_TYPE_A_CLOSURES(AST) (pkl_ast_type_resolv (AST)->type.val.array.closures)
+#define PKL_AST_TYPE_A_MAPPER(AST) (pkl_ast_type_resolv (AST)->type.val.array.closures[0])
+#define PKL_AST_TYPE_A_WRITER(AST) (pkl_ast_type_resolv (AST)->type.val.array.closures[1])
+#define PKL_AST_TYPE_A_BOUNDER(AST) (pkl_ast_type_resolv (AST)->type.val.array.closures[2])
+#define PKL_AST_TYPE_A_CONSTRUCTOR(AST) (pkl_ast_type_resolv (AST)->type.val.array.closures[3])
+#define PKL_AST_TYPE_A_INTEGRATOR(AST) (pkl_ast_type_resolv (AST)->type.val.array.closures[4])
+#define PKL_AST_TYPE_S_NFIELD(AST) (pkl_ast_type_resolv (AST)->type.val.sct.nfield)
+#define PKL_AST_TYPE_S_NCFIELD(AST) (pkl_ast_type_resolv (AST)->type.val.sct.ncfield)
+#define PKL_AST_TYPE_S_NDECL(AST) (pkl_ast_type_resolv (AST)->type.val.sct.ndecl)
+#define PKL_AST_TYPE_S_NELEM(AST) (pkl_ast_type_resolv (AST)->type.val.sct.nelem)
+#define PKL_AST_TYPE_S_ELEMS(AST) (pkl_ast_type_resolv (AST)->type.val.sct.elems)
+#define PKL_AST_TYPE_S_PINNED_P(AST) (pkl_ast_type_resolv (AST)->type.val.sct.pinned_p)
+#define PKL_AST_TYPE_S_UNION_P(AST) (pkl_ast_type_resolv (AST)->type.val.sct.union_p)
+#define PKL_AST_TYPE_S_CLOSURES(AST) (pkl_ast_type_resolv (AST)->type.val.sct.closures)
+#define PKL_AST_TYPE_S_MAPPER(AST) (pkl_ast_type_resolv (AST)->type.val.sct.closures[0])
+#define PKL_AST_TYPE_S_WRITER(AST) (pkl_ast_type_resolv (AST)->type.val.sct.closures[1])
+#define PKL_AST_TYPE_S_CONSTRUCTOR(AST) (pkl_ast_type_resolv (AST)->type.val.sct.closures[2])
+#define PKL_AST_TYPE_S_COMPARATOR(AST) (pkl_ast_type_resolv (AST)->type.val.sct.closures[3])
+#define PKL_AST_TYPE_S_INTEGRATOR(AST) (pkl_ast_type_resolv (AST)->type.val.sct.closures[4])
+#define PKL_AST_TYPE_S_DEINTEGRATOR(AST) (pkl_ast_type_resolv (AST)->type.val.sct.closures[5])
+#define PKL_AST_TYPE_S_TYPIFIER(AST) (pkl_ast_type_resolv (AST)->type.val.sct.closures[6])
+#define PKL_AST_TYPE_S_ITYPE(AST) (pkl_ast_type_resolv (AST)->type.val.sct.itype)
+#define PKL_AST_TYPE_O_UNIT(AST) (pkl_ast_type_resolv (AST)->type.val.off.unit)
+#define PKL_AST_TYPE_O_BASE_TYPE(AST) (pkl_ast_type_resolv (AST)->type.val.off.base_type)
+#define PKL_AST_TYPE_O_REF_TYPE(AST) (pkl_ast_type_resolv (AST)->type.val.off.ref_type)
+#define PKL_AST_TYPE_F_RTYPE(AST) (pkl_ast_type_resolv (AST)->type.val.fun.rtype)
+#define PKL_AST_TYPE_F_NARG(AST) (pkl_ast_type_resolv (AST)->type.val.fun.narg)
+#define PKL_AST_TYPE_F_ARGS(AST) (pkl_ast_type_resolv (AST)->type.val.fun.args)
+#define PKL_AST_TYPE_F_VARARG(AST) (pkl_ast_type_resolv (AST)->type.val.fun.vararg)
+#define PKL_AST_TYPE_F_FIRST_OPT_ARG(AST) (pkl_ast_type_resolv (AST)->type.val.fun.first_opt_arg)
 
 #define PKL_AST_TYPE_COMPLETE_UNKNOWN 0
 #define PKL_AST_TYPE_COMPLETE_YES 1
@@ -1001,14 +1019,21 @@ struct pkl_ast_type
 {
   struct pkl_ast_common common;
 
-  union pkl_ast_node *name;
   enum pkl_ast_type_code code;
   int complete;
   int compiled;
   int fallible;
+  int named_p;
 
   union
   {
+    struct
+    {
+      union pkl_ast_node *name;
+      union pkl_ast_node *type;
+      union pkl_ast_node *next_in_decl;
+    } alias;
+
     struct
     {
       size_t size;
@@ -1050,6 +1075,7 @@ struct pkl_ast_type
 
     struct
     {
+      union pkl_ast_node *name;
       union pkl_ast_node *rtype;
       int narg;
       int vararg;
@@ -1060,7 +1086,10 @@ struct pkl_ast_type
   } val;
 };
 
-pkl_ast_node pkl_ast_make_named_type (pkl_ast ast, pkl_ast_node name);
+pkl_ast_node pkl_ast_type_resolv (pkl_ast_node type);
+
+pkl_ast_node pkl_ast_make_named_type (pkl_ast ast, pkl_ast_node name,
+                                      pkl_ast_node type);
 
 pkl_ast_node pkl_ast_make_integral_type (pkl_ast ast, size_t size, int signed_p);
 
@@ -1158,6 +1187,9 @@ pkl_ast_node pkl_ast_type_incr_step (pkl_ast ast, pkl_ast_node type);
    re-declared non-immutable global entities in a compilation session.
    See pkl-env to see how this field is used.
 
+   TYPE_NAMES is a list of alias PKL_AST_TYPE nodes, whose names shall
+   be updated should this declaration be renamed.
+
    ORDER is the order of the declaration in its containing
    compile-time environment.  It is filled up when the declaration is
    registered in an environment.
@@ -1185,6 +1217,7 @@ pkl_ast_node pkl_ast_type_incr_step (pkl_ast ast, pkl_ast_node type);
 #define PKL_AST_DECL_IN_STRUCT_P(AST) ((AST)->decl.in_struct_p)
 #define PKL_AST_DECL_IMMUTABLE_P(AST) ((AST)->decl.immutable_p)
 #define PKL_AST_DECL_REDECL_CHAIN(AST) ((AST)->decl.redecl_chain)
+#define PKL_AST_DECL_TYPE_NAMES(AST) ((AST)->decl.type_names)
 
 #define PKL_AST_DECL_KIND_ANY 0
 #define PKL_AST_DECL_KIND_VAR 1
@@ -1205,6 +1238,7 @@ struct pkl_ast_decl
   union pkl_ast_node *initial;
   union pkl_ast_node *prev_decl;
   union pkl_ast_node *redecl_chain;
+  union pkl_ast_node *type_names;
   int order;
 };
 
@@ -1212,6 +1246,9 @@ pkl_ast_node pkl_ast_make_decl (pkl_ast ast, int kind,
                                 pkl_ast_node name, pkl_ast_node initial,
                                 const char *source);
 
+void pkl_ast_add_type_name_to_decl (pkl_ast_node decl, pkl_ast_node type_name);
+void pkl_ast_rename_decl (pkl_ast ast, pkl_ast_node decl, const char *new_name);
+
 /* PKL_AST_OFFSET nodes represent poke object constructions.
 
    MAGNITUDE is an integer expression.
@@ -1683,34 +1720,24 @@ pkl_ast_node pkl_ast_make_incrdecr (pkl_ast ast,
 
    STMTS is a possibly empty chain of statements.
 
-   If BUILTIN is not PKL_AST_BUILTIN_NONE, then this compound
-   statement is a compiler builtin, i.e. specific code will be
-   generated for this node.  In this case, STMTS should be NULL.
-
    If FRAMELESS_P is true then the compound statement will not use its
    own lexical frame.  */
 
 #define PKL_AST_COMP_STMT_STMTS(AST) ((AST)->comp_stmt.stmts)
-#define PKL_AST_COMP_STMT_BUILTIN(AST) ((AST)->comp_stmt.builtin)
 #define PKL_AST_COMP_STMT_NUMVARS(AST) ((AST)->comp_stmt.numvars)
 #define PKL_AST_COMP_STMT_FRAMELESS_P(AST) ((AST)->comp_stmt.frameless_p)
 
-#define PKL_AST_BUILTIN_NONE 0
-
 struct pkl_ast_comp_stmt
 {
   struct pkl_ast_common common;
 
   union pkl_ast_node *stmts;
-  int builtin;
   int numvars;
   int frameless_p;
 };
 
 pkl_ast_node pkl_ast_make_comp_stmt (pkl_ast ast, pkl_ast_node stmts);
 
-pkl_ast_node pkl_ast_make_builtin (pkl_ast ast, int builtin);
-
 /* PKL_AST_NULL_STMT nodes represent the "null statement".  It can
    appear anywhere an statement is expected, but it has no effect.  */
 
@@ -2017,6 +2044,7 @@ pkl_ast_node pkl_ast_make_print_stmt (pkl_ast ast,
 #define PKL_AST_BREAK_CONTINUE_STMT_KIND(AST) ((AST)->break_continue_stmt.kind)
 #define PKL_AST_BREAK_CONTINUE_STMT_ENTITY(AST) ((AST)->break_continue_stmt.entity)
 #define PKL_AST_BREAK_CONTINUE_STMT_NFRAMES(AST) ((AST)->break_continue_stmt.nframes)
+#define PKL_AST_BREAK_CONTINUE_STMT_NPOPES(AST) ((AST)->break_continue_stmt.npopes)
 
 #define PKL_AST_BREAK_CONTINUE_STMT_KIND_BREAK 0
 #define PKL_AST_BREAK_CONTINUE_STMT_KIND_CONTINUE 1
@@ -2027,6 +2055,7 @@ struct pkl_ast_break_continue_stmt
   union pkl_ast_node *entity;
   int kind;
   int nframes;
+  int npopes;
 };
 
 pkl_ast_node pkl_ast_make_break_continue_stmt (pkl_ast ast,
