@@ -832,7 +832,32 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_op_binary)
   int restart1, restart2;
 
   pkl_ast_node exp = PKL_PASS_NODE;
-  pkl_ast_node type = PKL_AST_TYPE (exp);
+  pkl_ast_node type;
+
+  switch (PKL_AST_EXP_CODE (exp))
+    {
+    case PKL_AST_OP_AND:
+    case PKL_AST_OP_OR:
+    case PKL_AST_OP_IMPL:
+      {
+        pkl_ast_node op1_type;
+        pkl_ast_node op2_type;
+
+        op1_type = PKL_AST_TYPE (PKL_AST_EXP_OPERAND (exp, 0));
+        op2_type = PKL_AST_TYPE (PKL_AST_EXP_OPERAND (exp, 1));
+
+        if (PKL_AST_TYPE_CODE (op1_type) == PKL_TYPE_STRUCT)
+          op1_type = PKL_AST_TYPE_S_ITYPE (op1_type);
+        if (PKL_AST_TYPE_CODE (op2_type) == PKL_TYPE_STRUCT)
+          op2_type = PKL_AST_TYPE_S_ITYPE (op2_type);
+
+        /* Promote operands to a type that is wide enough for both of them.  */
+        type = pkl_type_integral_promote (PKL_PASS_AST, op1_type, op2_type);
+      }
+      break;
+    default:
+      type = PKL_AST_TYPE (exp);
+    }
 
   if (PKL_AST_TYPE_CODE (type) == PKL_TYPE_INTEGRAL)
     {
@@ -869,8 +894,19 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_op_unary)
 
   if (PKL_AST_TYPE_CODE (type) == PKL_TYPE_INTEGRAL)
     {
-      size_t size = PKL_AST_TYPE_I_SIZE (type);
-      int signed_p = PKL_AST_TYPE_I_SIGNED_P (type);
+      size_t size;
+      int signed_p;
+
+      if (PKL_AST_EXP_CODE (node) == PKL_AST_OP_NOT)
+        {
+          pkl_ast_node exp_type = PKL_AST_TYPE (PKL_AST_EXP_OPERAND (node, 0));
+
+          if (PKL_AST_TYPE_CODE (exp_type) == PKL_TYPE_STRUCT)
+            type = PKL_AST_TYPE_S_ITYPE (exp_type);
+        }
+
+      size = PKL_AST_TYPE_I_SIZE (type);
+      signed_p = PKL_AST_TYPE_I_SIGNED_P (type);
 
       if (!promote_integral (PKL_PASS_AST, size, signed_p,
                              &PKL_AST_EXP_OPERAND (node, 0), &restart))
@@ -1191,28 +1227,35 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_if_stmt)
 }
 PKL_PHASE_END_HANDLER
 
-/* The condition in loop statements, if present, shall be promoted to
-   int<32>.  */
+/* Promote integral structs to their itype when used in condition of a
+   FOR-IN-WHERE statement.  The rest will be taken care of by the code
+   generator.  */
 
 PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_loop_stmt)
 {
   pkl_ast_node loop_stmt = PKL_PASS_NODE;
-  pkl_ast_node condition = PKL_AST_LOOP_STMT_CONDITION (loop_stmt);
+  pkl_ast_node cond = PKL_AST_LOOP_STMT_CONDITION (loop_stmt);
+  pkl_ast_node cond_type;
 
-  if (condition)
+  if (cond == NULL)
+    PKL_PASS_DONE;
+
+  cond_type = PKL_AST_TYPE (cond);
+  if (PKL_AST_TYPE_CODE (cond_type) == PKL_TYPE_STRUCT
+      && PKL_AST_TYPE_S_ITYPE (cond_type))
     {
+      pkl_ast_node itype = PKL_AST_TYPE_S_ITYPE (cond_type);
+      size_t size = PKL_AST_TYPE_I_SIZE (itype);
+      int signed_p = PKL_AST_TYPE_I_SIGNED_P (itype);
       int restart;
 
-      /* Note that the condition node is promoteable as per
-         typify.  */
-
       if (!promote_integral (PKL_PASS_AST,
-                             32, 1,
+                             size, signed_p,
                              &PKL_AST_LOOP_STMT_CONDITION (loop_stmt),
                              &restart))
         {
           PKL_ICE (PKL_AST_LOC (loop_stmt),
-                   "couldn't promote condition of lop-stmt #%" PRIu64,
+                   "couldn't promote condition of loop-stmt #%" PRIu64,
                    PKL_AST_UID (loop_stmt));
           PKL_PASS_ERROR;
         }
